// Copyright (C) 2010-2016  Lukas Lalinsky
// Distributed under the MIT license, see the LICENSE file for details.

#ifndef CHROMAPRINT_FFT_FRAME_H_
#define CHROMAPRINT_FFT_FRAME_H_

#include <vector>

namespace chromaprint {

typedef std::vector<double> FFTFrame;

}; // namespace chromaprint

#endif

